@@ -24,7 +24,7 @@
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -868,6 +868,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     bounds;
 
   size_t
+    imageListLength,
     length,
     page,
     pixel,
@@ -933,6 +934,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
   (void) memset(&bounds,0,sizeof(bounds));
   page=0;
   scene=0;
+  imageListLength=GetImageListLength(image);
   do
   {
     /*
@@ -1601,8 +1603,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=SetImageProgress(image,SaveImagesTag,scene++,
-      GetImageListLength(image));
+    status=SetImageProgress(image,SaveImagesTag,scene++,imageListLength);
     if (status == MagickFalse)
       break;
   } while (image_info->adjoin != MagickFalse);
