@@ -23,7 +23,7 @@
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -278,7 +278,7 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
       number_planes++;
     number_pixels=(MagickSizeType) image->columns*image->rows;
     if ((GetBlobSize(image) == 0) || ((((MagickSizeType) number_pixels*
-         number_planes*bits_per_pixel/8)/GetBlobSize(image)) > 254.0))
+         number_planes*bits_per_pixel/8)/GetBlobSize(image)) > 254))
       ThrowRLEException(CorruptImageError,"InsufficientImageDataInFile")
     if (((MagickSizeType) number_colormaps*map_length) > GetBlobSize(image))
       ThrowRLEException(CorruptImageError,"InsufficientImageDataInFile")
@@ -321,11 +321,13 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
             if (comment == (char *) NULL)
               ThrowRLEException(ResourceLimitError,"MemoryAllocationFailed");
             (void) ReadBlob(image,length-1,(unsigned char *) comment);
-            if (count == (length-1))
+            if (count != (length-1))
               {
-                comment[length-1]='\0';
-                (void) SetImageProperty(image,"comment",comment);
+                comment=DestroyString(comment);
+                ThrowRLEException(CorruptImageError,"UnexpectedEndOfFile");
               }
+            comment[length-1]='\0';
+            (void) SetImageProperty(image,"comment",comment);
             comment=DestroyString(comment);
             if ((length & 0x01) == 0)
               (void) ReadBlobByte(image);
@@ -702,8 +704,8 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AcquireNextImage(image_info,image);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
-            image=DestroyImageList(image);
-            return((Image *) NULL);
+            status=MagickFalse;
+            break;
           }
         image=SyncNextImageInList(image);
         status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
@@ -713,6 +715,8 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
   } while ((count != 0) && (memcmp(magick,"\122\314",2) == 0));
   (void) CloseBlob(image);
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   return(GetFirstImageInList(image));
 }
 
