@@ -23,7 +23,7 @@
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -291,9 +291,10 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   */
   cluster=(Cluster *) NULL;
   head=(Cluster *) NULL;
-  (void) ResetMagickMemory(&red,0,sizeof(red));
-  (void) ResetMagickMemory(&green,0,sizeof(green));
-  (void) ResetMagickMemory(&blue,0,sizeof(blue));
+  (void) memset(&red,0,sizeof(red));
+  (void) memset(&green,0,sizeof(green));
+  (void) memset(&blue,0,sizeof(blue));
+  exception=(&image->exception);
   while (DefineRegion(extrema[Red],&red) != 0)
   {
     green.index=0;
@@ -355,7 +356,6 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   status=MagickTrue;
   count=0;
   progress=0;
-  exception=(&image->exception);
   image_view=AcquireVirtualCacheView(image,exception);
   for (y=0; y < (ssize_t) image->rows; y++)
   {
@@ -533,7 +533,7 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   exception=(&image->exception);
   image_view=AcquireAuthenticCacheView(image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(progress,status) \
+  #pragma omp parallel for schedule(static) shared(progress,status) \
     magick_number_threads(image,image,image->rows,1)
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
@@ -1005,9 +1005,9 @@ MagickExport MagickBooleanType GetImageDynamicThreshold(const Image *image,
   */
   cluster=(Cluster *) NULL;
   head=(Cluster *) NULL;
-  (void) ResetMagickMemory(&red,0,sizeof(red));
-  (void) ResetMagickMemory(&green,0,sizeof(green));
-  (void) ResetMagickMemory(&blue,0,sizeof(blue));
+  (void) memset(&red,0,sizeof(red));
+  (void) memset(&green,0,sizeof(green));
+  (void) memset(&blue,0,sizeof(blue));
   while (DefineRegion(extrema[Red],&red) != 0)
   {
     green.index=0;
@@ -1827,7 +1827,7 @@ MagickExport MagickBooleanType SegmentImage(Image *image,
           extrema[i]=(short *) RelinquishMagickMemory(extrema[i]);
           histogram[i]=(ssize_t *) RelinquishMagickMemory(histogram[i]);
         }
-        ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        ThrowBinaryImageException(ResourceLimitError,"MemoryAllocationFailed",
           image->filename)
       }
   }
