@@ -23,7 +23,7 @@
 %  You may not use this file except in compliance with the license.  You may  %
 %  obtain a copy of the license at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %  unless required by applicable law or agreed to in writing, software        %
 %  distributed under the license is distributed on an "as is" basis,          %
 %  without warranties or conditions of any kind, either express or implied.   %
@@ -58,7 +58,12 @@
 #include "magick/utility-private.h"
 
 /*
-  static declarations.
+  Define declarations.
+*/
+#define CharsPerLine  0x14
+
+/*
+  Static declarations.
 */
 #if !defined(MAGICKCORE_HAVE_STRCASECMP) || !defined(MAGICKCORE_HAVE_STRNCASECMP)
 static const unsigned char
@@ -237,6 +242,8 @@ MagickExport StringInfo *BlobToStringInfo(const void *blob,const size_t length)
     }
   if (blob != (const void *) NULL)
     (void) memcpy(string_info->datum,blob,length);
+  else
+    (void) memset(string_info->datum,0,length*sizeof(*string_info->datum));
   (void) memset(string_info->datum+length,0,MagickPathExtent*
     sizeof(*string_info->datum));
   return(string_info);
@@ -1060,7 +1067,8 @@ MagickExport StringInfo *FileToStringInfo(const char *filename,
 %
 %  The format of the FormatMagickSize method is:
 %
-%      ssize_t FormatMagickSize(const MagickSizeType size,char *format)
+%      ssize_t FormatMagickSize(const MagickSizeType size,
+%        const MagickBooleanType by,char *format)
 %
 %  A description of each parameter follows:
 %
@@ -1591,16 +1599,16 @@ MagickExport void PrintStringInfo(FILE *file,const char *id,
     Convert string to a HEX list.
   */
   p=(char *) string_info->datum;
-  for (i=0; i < string_info->length; i+=0x14)
+  for (i=0; i < string_info->length; i+=CharsPerLine)
   {
-    (void) FormatLocaleFile(file,"0x%08lx: ",(unsigned long) (0x14*i));
-    for (j=1; j <= MagickMin(string_info->length-i,0x14); j++)
+    (void) FormatLocaleFile(file,"0x%08lx: ",(unsigned long) (CharsPerLine*i));
+    for (j=1; j <= MagickMin(string_info->length-i,CharsPerLine); j++)
     {
       (void) FormatLocaleFile(file,"%02lx",(unsigned long) (*(p+j)) & 0xff);
       if ((j % 0x04) == 0)
         (void) fputc(' ',file);
     }
-    for ( ; j <= 0x14; j++)
+    for ( ; j <= CharsPerLine; j++)
     {
       (void) fputc(' ',file);
       (void) fputc(' ',file);
@@ -1608,7 +1616,7 @@ MagickExport void PrintStringInfo(FILE *file,const char *id,
         (void) fputc(' ',file);
     }
     (void) fputc(' ',file);
-    for (j=1; j <= MagickMin(string_info->length-i,0x14); j++)
+    for (j=1; j <= MagickMin(string_info->length-i,CharsPerLine); j++)
     {
       if (isprint((int) ((unsigned char) *p)) != 0)
         (void) fputc(*p,file);
@@ -2206,6 +2214,8 @@ MagickExport double *StringToArrayOfDoubles(const char *string,ssize_t *count,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
   *count=0;
+  if (string == (char *) NULL)
+    return((double *) NULL);  /* no value found */
   i=0;
   p=string;
   while (*p != '\0')
@@ -2412,7 +2422,7 @@ MagickExport char **StringToList(const char *text)
       /*
         Convert string to a HEX list.
       */
-      lines=(size_t) (strlen(text)/0x14)+1;
+      lines=(size_t) (strlen(text)/CharsPerLine)+1;
       textlist=(char **) AcquireQuantumMemory((size_t) lines+1UL,
         sizeof(*textlist));
       if (textlist == (char **) NULL)
@@ -2425,9 +2435,9 @@ MagickExport char **StringToList(const char *text)
         if (textlist[i] == (char *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"UnableToConvertText");
         (void) FormatLocaleString(textlist[i],MaxTextExtent,"0x%08lx: ",
-          (long) (0x14*i));
+          (long) (CharsPerLine*i));
         q=textlist[i]+strlen(textlist[i]);
-        for (j=1; j <= (ssize_t) MagickMin(strlen(p),0x14); j++)
+        for (j=1; j <= (ssize_t) MagickMin(strlen(p),CharsPerLine); j++)
         {
           (void) FormatLocaleString(hex_string,MaxTextExtent,"%02x",*(p+j));
           (void) CopyMagickString(q,hex_string,MaxTextExtent);
@@ -2435,7 +2445,7 @@ MagickExport char **StringToList(const char *text)
           if ((j % 0x04) == 0)
             *q++=' ';
         }
-        for ( ; j <= 0x14; j++)
+        for ( ; j <= CharsPerLine; j++)
         {
           *q++=' ';
           *q++=' ';
@@ -2443,7 +2453,7 @@ MagickExport char **StringToList(const char *text)
             *q++=' ';
         }
         *q++=' ';
-        for (j=1; j <= (ssize_t) MagickMin(strlen(p),0x14); j++)
+        for (j=1; j <= (ssize_t) MagickMin(strlen(p),CharsPerLine); j++)
         {
           if (isprint((int) ((unsigned char) *p)) != 0)
             *q++=(*p);
@@ -2452,6 +2462,10 @@ MagickExport char **StringToList(const char *text)
           p++;
         }
         *q='\0';
+        textlist[i]=(char *) ResizeQuantumMemory(textlist[i],q-textlist[i]+1,
+          sizeof(**textlist));
+        if (textlist[i] == (char *) NULL)
+          ThrowFatalException(ResourceLimitFatalError,"UnableToConvertText");
       }
     }
   textlist[i]=(char *) NULL;
